#include <stdio.h>
#include <gnugetopt/getopt.h>
#include "parser.h"
#include "writer.h"

#define OPT_STR 		"78edh"

int helpText(char *cmd) {
	fprintf(stderr, "%s -e|-d [-7 | -8] [-h] [input_filename]\n", cmd);
	fprintf(stderr, " -e   encode to SBC\n");
	fprintf(stderr, " -d   decode from SBC\n");
	fprintf(stderr, " -7   7-bit input/output\n");
	fprintf(stderr, " -8   8-bit input/output (default)\n");
	fprintf(stderr, " -h   This help text\n");
	fprintf(stderr, "If input_filename is not given stdin is assumed\n");
	fprintf(stderr, "Output is always on stdout\n");
	return 1;
}

int main(int argc, char **argv) {
	int c;
	int bit7 = 0;
	int bit8 = 0;
	int encode = 0;
	int decode = 0;
	FILE *in;
	while ((c = getopt(argc, argv, OPT_STR)) != -1) {
		switch(c) {
			case '7':
				bit7 = 1;
				break;
			case '8':
				bit8 = 1;
				break;
			case 'e':
				encode = 1;
				break;
			case 'd':
				decode = 1;
				break;
			case 'h':
				return helpText(argv[0]);
			case '?':
			default:
				return helpText(argv[0]);
		}
	}
	// Sanity check the arguments...
	if ((encode == 0) && (decode == 0)) {
		fprintf(stderr, "Must select encode or decode\n");
		return helpText(argv[0]);
	}
	if ((encode == 1) && (decode == 1)) {
		fprintf(stderr, "Cannot select encode and decode\n");
		return helpText(argv[0]);
	}
	if ((bit7 == 0) && (bit8 == 0))
		bit8 = 1;
	if ((bit7 == 1) && (bit8 == 1)) {
		fprintf(stderr, "Cannot select 7-bit and 8-bit\n");
		return helpText(argv[0]);
	}
	argc -= optind;
	argv += optind;
	// Open stdin or file from command line...
	if (argc == 0) {
		in = stdin;
	}else{
		in = fopen(argv[0], "rb");
		if (in == 0) {
			fprintf(stderr, "Unable to open file %s\n", argv[0]);
			return 1;
		}
	}
	if (encode == 1) {
		write(in, stdout, bit7?BITS_7:BITS_8);
	}else{
		read(in, stdout, bit7?BITS_7:BITS_8);
	}
	fclose(in);
	return 0;
}
