#include <stdio.h>
#include "parser.h"

int PARSER_LINE;
int PARSER_COLUMN;
int BAD_CHAR_CODE;


int parser_mode;

/**
 * Initialize the parser variables.  This needs to be called once per file
 * parsed.
 */
void init() {
	PARSER_LINE = 1;
	PARSER_COLUMN = 0;
	parser_mode = 99;
}

/**
 * Is the given character whitespace?  Whitespace is defined as space,
 * tab, carriage return, and linefeed.
 */
int isWhitespace(int c) {
	return (c == ' ') || (c == '\t') || (c == '\r') || (c == '\n');
}

/**
 * Is the given character a newline?  A newline is defined as only a linefeed
 * because both Unix and Windows use that (windows prefixes it with a
 * carriage return, but that gets treated as whitespace)
 */
int isNewline(int c) {
	return c == '\n';
}

/**
 * Get the next token from the input file.  Possible return values are:
 * TOKEN_SPAM for "spam"
 * TOKEN_BOINK for "boink"
 * TOKEN_CHEESE for "cheese"
 * TOKEN_EOF for an expected EOF (that is, an EOF after the final token)
 * PREMATURE_EOF an unexpected EOF (EOF that occurs while parsing a token)
 * BAD_CHAR for an unexpected character
 *
 * The error return values (premature EOF and bad character) will leave
 * the three variables PARSER_LINE, PARSER_COLUMN, and BAD_CHAR_CODE
 * populated with relevant info.
 *
 * This is implemented as a finite state machine.
 * At some point this should probably be rewritten with lex/yacc.
 */
int getNextToken(FILE *in) {
	int c;
	while (1) {
		// Check for end of file
		if (feof(in)) {
			if (parser_mode == 99)
				return TOKEN_EOF;
			else
				return PREMATURE_EOF;
		}
		// Get char
		c = fgetc(in);
		BAD_CHAR_CODE = c;
		if (c == EOF) {
			if (parser_mode == 99)
				return TOKEN_EOF;
			else
				return PREMATURE_EOF;
		}
		// To lowercase
		if ((c >= 'A') && (c <= 'Z'))
			c = c - 'A' + 'a';
		PARSER_COLUMN++;
		switch(parser_mode) {
			case 99:
				if (isNewline(c)) {
					PARSER_COLUMN = 0;
					PARSER_LINE++;
					continue;
				}else if (isWhitespace(c)){
					continue;
				}else if (c == 's'){
					parser_mode = 1;
				}else if (c == 'b'){
					parser_mode = 5;
				}else if (c == 'c'){
					parser_mode = 10;
				}else{
					return BAD_CHAR;
				}
				break;
			case 1:
				if (c == 'p')
					parser_mode = 2;
				else
					return BAD_CHAR;
				break;
			case 2:
				if (c == 'a')
					parser_mode = 3;
				else
					return BAD_CHAR;
				break;
			case 3:
				if (c == 'm') {
					parser_mode = 99;
					return TOKEN_SPAM;
				}else
					return BAD_CHAR;
				break;
			case 5:
				if (c == 'o')
					parser_mode = 6;
				else
					return BAD_CHAR;
				break;
			case 6:
				if (c == 'i')
					parser_mode = 7;
				else
					return BAD_CHAR;
				break;
			case 7:
				if (c == 'n')
					parser_mode = 8;
				else
					return BAD_CHAR;
				break;
			case 8:
				if (c == 'k') {
					parser_mode = 99;
					return TOKEN_BOINK;
				} else
					return BAD_CHAR;
				break;
			case 10:
				if (c == 'h')
					parser_mode = 11;
				else
					return BAD_CHAR;
				break;
			case 11:
			case 12:
				if (c == 'e')
					parser_mode++;
				else
					return BAD_CHAR;
				break;
			case 13:
				if (c == 's')
					parser_mode = 14;
				else
					return BAD_CHAR;
				break;
			case 14:
				if (c == 'e') {
					parser_mode = 99;
					return TOKEN_CHEESE;
				} else
					return BAD_CHAR;
				break;
		}
	}
	return 0;
}
