#include "reader.h"
#include "parser.h"

/**
 * Read an input file full of Spam/Boink/Cheese data and output the original
 * raw data.  Proper EOF is defined as four nulls (cheeses).  Both 7-bit and
 * 8-bit input is supported (in fact, an error is generated if the input data
 * is not the proper number of bits).
 *
 * Output, in the form of binary (typically ASCII) is written to the "out" 
 * file handle.
 */
void read(FILE *in, FILE *out, int bits) {
	int token;
	int c = 0;
	int bitCount = 0;
	int cheeseCount = 0;
	init();
	while (1) {
		// Parse the next token
		token = getNextToken(in);
		switch(token) {
			case TOKEN_EOF:
				return;
			case BAD_CHAR:
				fprintf(stderr, "Bad character '%c' found at line %d, column %d (parser mode %d)\n", BAD_CHAR_CODE, PARSER_LINE, PARSER_COLUMN, parser_mode);
				return;
			case PREMATURE_EOF:
				fprintf(stderr, "Premature end of file found at line %d, column %d\n", PARSER_LINE, PARSER_COLUMN);
				return;
			case TOKEN_BOINK:
#ifdef DEBUG
				printf("Got token BOINK\n");
#endif				
				cheeseCount = 0;
				c = c << 1;
				bitCount++;
				break;
			case TOKEN_SPAM:
#ifdef DEBUG
				printf("Got token SPAM\n");
#endif
				cheeseCount = 0;
				c = c << 1;
				c = c | 0x01;
				bitCount++;
				break;
			case TOKEN_CHEESE:
#ifdef DEBUG
				printf("Got token CHEESE (count==%d)\n", cheeseCount);
#endif
				if (cheeseCount == 0) {
					if (bitCount != bits)
						fprintf(stderr, "Warning: incorrect number of bits.  Got %d, expected %d.\n", bitCount, bits);
					fprintf(out, "%c", c);
				}
				c = 0;
				bitCount = 0;
				cheeseCount++;
				if (cheeseCount >= 4)
					return;
				break;
			default:
				fprintf(stderr, "Unknown token (%d)\n", token);
		}
	}
}
